%--------------------------------------------------------------------------
% Load the data: 
%--------------------------------------------------------------------------
load ratings.mat;

m = size(D, 1); % number of movies
n = size(D, 2); % number of users

%--------------------------------------------------------------------------
% Assignment a)
%--------------------------------------------------------------------------
% This would do the job, however we can do it more efficiently
% [U,S,V] = svd(D, 'econ');

% We only need an approximation to the column space. So rather than
% computing the complete SVD, we only compute the eigenvalue decomposition
% of the m-by-m matrix: 
%       D*D' = U*S^2*U'
% which is much more efficient in this case because m << n:
[U, E] = eig(D*D');
% Reorder elements since ordering of eig is opposite to ordering of svd:
U = U(:,end:-1:1);
E = diag(E);
S = diag(sqrt(E(end:-1:1)));

figure(); 
subplot(1,3,1); 
plot(1:m, diag(S), 'r-', 'LineWidth', 2);
set(get(gca, 'YLabel'), 'String', '\sigma');
set(get(gca, 'YLabel'), 'FontSize', 20);
set(get(gca, 'XLabel'), 'String', 'r');
set(get(gca, 'XLabel'), 'FontSize', 15);

subplot(1,3,2); 
plot(1:m, log(diag(S)), 'r-', 'LineWidth', 2);
set(get(gca, 'YLabel'), 'String', 'log(\sigma)');
set(get(gca, 'YLabel'), 'FontSize', 20);
set(get(gca, 'XLabel'), 'String', 'r');
set(get(gca, 'XLabel'), 'FontSize', 15);

subplot(1,3,3); 
plot(1:m, log(diag(S)), 'r-', 'LineWidth', 2);
set(get(gca, 'YLabel'), 'String', 'log(\sigma)');
set(get(gca, 'YLabel'), 'FontSize', 20);
set(get(gca, 'XLabel'), 'String', 'log(r)');
set(get(gca, 'XLabel'), 'FontSize', 15);
set(gca, 'XScale', 'log')

% According to the plots, rank-10 approximation should be accurate enough
r = 10;
Ur = U(:,1:r);
Sr = S(1:r,1:r);
